import React from 'react';
import { ViewProps } from 'react-native';
import type { FUNSDK_DOWNLOAD_STATE_ENUM, FUNSDK_MEDIA_PLAY_STATE_ENUM } from '../types/stream';
import type { SearchDate } from '../manager';
export declare enum SearcResultRecordEnum {
    RECORD_FILES = "recordfiles",
    RECORD_TIMELIST = "recordtimelist"
}
export type RecordViewNativeProps = ViewProps & {
    onMediaPlayState: (event: {
        nativeEvent: {
            playSpeed: number;
            state: number;
        };
    }) => void;
    onShowRateAndTime: (event: {
        nativeEvent: {
            time: string;
            rate: string;
        };
    }) => void;
    onVideoBufferEnd: (event: {
        nativeEvent: {
            isBufferEnd: boolean;
        };
    }) => void;
    onFailed: (event: {
        nativeEvent: {
            msgId: number;
            errorId: number;
        };
    }) => void;
    onStartInit: (event: {
        nativeEvent: {
            status: 'start' | 'initialized';
        };
    }) => void;
    onDebugState: (event: {
        nativeEvent: {
            state: string;
        };
    }) => void;
    onDownloadProgress?: (event: {
        nativeEvent: {
            progress: number;
        };
    }) => void;
    onDownloadState?: (event: {
        nativeEvent: {
            downloadState: FUNSDK_DOWNLOAD_STATE_ENUM;
            fileName: string;
        };
    }) => void;
};
export type RecordViewComponent = React.ComponentClass<RecordViewNativeProps>;
export declare const RecordView: import("react-native").HostComponent<RecordViewNativeProps>;
export type RecordPlayerProps = ViewProps & {
    devId: string;
    channelId: number;
    onMediaPlayState?: (obj: {
        playSpeed: number;
        state: FUNSDK_MEDIA_PLAY_STATE_ENUM;
    }) => void;
    onShowRateAndTime?: (obj: {
        time: string;
        rate: string;
    }) => void;
    onVideoBufferEnd?: (obj: {
        isBufferEnd: boolean;
    }) => void;
    onFailed?: (obj: {
        msgId: number;
        errorId: number;
    }) => void;
    onStartInit?: (obj: {
        status: 'start' | 'initialized';
    }) => void;
    onDebugState?: (obj: {
        state: string;
    }) => void;
    onDownloadProgress?: (obj: {
        progress: number;
    }) => void;
    onDownloadState?: (obj: {
        downloadState: FUNSDK_DOWNLOAD_STATE_ENUM;
        fileName: string;
    }) => void;
};
export type RecordPlayerRef = {
    init: () => void;
    startPlayRecordByTime: (start: SearchDate, end: SearchDate) => void;
    capture: (path: string) => void;
    startRecord: (path: string) => void;
    stopRecord: () => void;
    isRecording: () => void;
    openVoice: () => void;
    closeVoice: () => void;
    pausePlay: () => void;
    rePlay: () => void;
    stopPlay: () => void;
    destroyPlay: () => void;
    isRecordPlay: () => void;
    downloadFile: (position: number, path: string) => void;
    setPlaySpeed: (playSpeed: 0 | 1 | 2 | number) => void;
};
export declare const RecordPlayer: React.ForwardRefExoticComponent<ViewProps & {
    devId: string;
    channelId: number;
    onMediaPlayState?: ((obj: {
        playSpeed: number;
        state: FUNSDK_MEDIA_PLAY_STATE_ENUM;
    }) => void) | undefined;
    onShowRateAndTime?: ((obj: {
        time: string;
        rate: string;
    }) => void) | undefined;
    onVideoBufferEnd?: ((obj: {
        isBufferEnd: boolean;
    }) => void) | undefined;
    onFailed?: ((obj: {
        msgId: number;
        errorId: number;
    }) => void) | undefined;
    onStartInit?: ((obj: {
        status: 'start' | 'initialized';
    }) => void) | undefined;
    onDebugState?: ((obj: {
        state: string;
    }) => void) | undefined;
    onDownloadProgress?: ((obj: {
        progress: number;
    }) => void) | undefined;
    onDownloadState?: ((obj: {
        downloadState: FUNSDK_DOWNLOAD_STATE_ENUM;
        fileName: string;
    }) => void) | undefined;
} & React.RefAttributes<RecordPlayerRef>>;
//# sourceMappingURL=index.d.ts.map